/**
 * @file hv_drv_UsbFatfsSystem.c
 * @brief Defines the apis depends os.
 * @details Sample Code of OS Dependent Functions for FatFs.
 * @author HiView SoC Software Team
 * @version 0.0.1
 * @date 20230417
 * @copyright Copyright(c),2022-9, Hiview Software. All rights reserved.
 * @par History:
 * <table>
 * <tr><th>Author                         <th>Date             <th>Details
 * <tr><td>HiView SoC Software Team   <td>20230417       <td>Create and first Version.
 * </table>
 **/
 
/* FreeRTOS kernel includes. */
#include "FreeRTOS.h"
#include "semphr.h"
#include "task.h"

#include "hv_drv_UsbFatfs.h"

#if FF_USE_LFN == 3 /* Dynamic memory allocation */

/*------------------------------------------------------------------------*/
/* Allocate a memory block                                                */
/*------------------------------------------------------------------------*/

void *ff_memalloc(           /* Returns pointer to the allocated memory block (null if not enough core) */
                  UINT msize /* Number of bytes to allocate */
)
{
    return HV_MALLOC(msize); /* Allocate a new memory block with POSIX API */
}

/*------------------------------------------------------------------------*/
/* Free a memory block                                                    */
/*------------------------------------------------------------------------*/

void ff_memfree(
    void *mblock /* Pointer to the memory block to free (nothing to do if null) */
)
{
    HV_FREE(mblock); /* Free the memory block with POSIX API */
}

#endif

#if FF_FS_REENTRANT /* Mutal exclusion */

/*------------------------------------------------------------------------*/
/* Create a Synchronization Object                                        */
/*------------------------------------------------------------------------*/
/* This function is called in f_mount() function to create a new
/  synchronization object for the volume, such as semaphore and mutex.
/  When a 0 is returned, the f_mount() function fails with FR_INT_ERR.
*/

//const osMutexDef_t Mutex[FF_VOLUMES];    /* Table of CMSIS-RTOS mutex */

int ff_cre_syncobj(                /* 1:Function succeeded, 0:Could not create the sync object */
                   BYTE       vol, /* Corresponding volume (logical drive number) */
                   FF_SYNC_t *sobj /* Pointer to return the created sync object */
)
{
    /* FreeRTOS */
    *sobj = xSemaphoreCreateMutex();
    return (int)(*sobj != NULL);
}

/*------------------------------------------------------------------------*/
/* Delete a Synchronization Object                                        */
/*------------------------------------------------------------------------*/
/* This function is called in f_mount() function to delete a synchronization
/  object that created with ff_cre_syncobj() function. When a 0 is returned,
/  the f_mount() function fails with FR_INT_ERR.
*/

int ff_del_syncobj(               /* 1:Function succeeded, 0:Could not delete due to an error */
                   FF_SYNC_t sobj /* Sync object tied to the logical drive to be deleted */
)
{
    /* FreeRTOS */
    vSemaphoreDelete(sobj);
    return 1;
}

/*------------------------------------------------------------------------*/
/* Request Grant to Access the Volume                                     */
/*------------------------------------------------------------------------*/
/* This function is called on entering file functions to lock the volume.
/  When a 0 is returned, the file function fails with FR_TIMEOUT.
*/

int ff_req_grant(               /* 1:Got a grant to access the volume, 0:Could not get a grant */
                 FF_SYNC_t sobj /* Sync object to wait */
)
{
    /* FreeRTOS */
    return (int)(xSemaphoreTake(sobj, FF_FS_TIMEOUT) == pdHV_TRUE);
}

/*------------------------------------------------------------------------*/
/* Release Grant to Access the Volume                                     */
/*------------------------------------------------------------------------*/
/* This function is called on leaving file functions to unlock the volume.
*/

void ff_rel_grant(
    FF_SYNC_t sobj /* Sync object to be signaled */
)
{
    /* FreeRTOS */
    xSemaphoreGive(sobj);
}

#endif
